@@ -1,6 +1,7 @@
 #! /bin/sh
-# $Id: iptables_init.sh,v 1.8 2014/04/15 13:45:08 nanard Exp $
+# $Id: iptables_init.sh,v 1.10 2017/04/21 11:16:09 nanard Exp $
 IPTABLES="`which iptables`" || exit 1
+IPTABLES="$IPTABLES -w"
 IP="`which ip`" || exit 1
 
 #change this parameters :
@@ -26,5 +27,5 @@ $IPTABLES -t filter -N MINIUPNPD
 $IPTABLES -t filter -A FORWARD -i $EXTIF ! -o $EXTIF -j MINIUPNPD
 
 #adding the MINIUPNPD chain for nat
-$IPTABLES -t nat -N MINIUPNPD-PCP-PEER
-$IPTABLES -t nat -A POSTROUTING -o $EXTIF -j MINIUPNPD-PCP-PEER
+$IPTABLES -t nat -N MINIUPNPD-POSTROUTING
+$IPTABLES -t nat -A POSTROUTING -o $EXTIF -j MINIUPNPD-POSTROUTING
