@@ -1,7 +1,7 @@
-/* $Id: iptpinhole.h,v 1.9 2014/10/21 14:07:20 nanard Exp $ */
+/* $Id: iptpinhole.h,v 1.10 2016/01/19 10:03:30 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2012 Thomas Bernard
+ * (c) 2012-2016 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 #ifndef IPTPINHOLE_H_INCLUDED
@@ -10,6 +10,12 @@
 #ifdef ENABLE_UPNPPINHOLE
 #include <sys/types.h>
 
+int find_pinhole(const char * ifname,
+                 const char * rem_host, unsigned short rem_port,
+                 const char * int_client, unsigned short int_port,
+                 int proto,
+                 char *desc, int desc_len, unsigned int * timestamp);
+
 int add_pinhole(const char * ifname,
                 const char * rem_host, unsigned short rem_port,
                 const char * int_client, unsigned short int_port,
